@@ -13,9 +13,7 @@
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
 from typing import Any
-from typing import Optional
 from typing import TextIO
-from typing import Union
 
 import cProfile
 import json
@@ -33,6 +31,7 @@ from nbxmpp.const import ConnectionType
 from nbxmpp.http import HTTPRequest
 from packaging.version import Version as V
 
+from gajim import IS_PORTABLE
 from gajim.common import app
 from gajim.common import configpaths
 from gajim.common import ged
@@ -59,8 +58,6 @@ from gajim.common.storage.draft import DraftStorage
 from gajim.common.storage.events import EventStorage
 from gajim.common.task_manager import TaskManager
 from gajim.common.util.http import create_http_request
-from gajim.plugins.pluginmanager import PluginManager
-from gajim.plugins.repository import PluginRepository
 
 
 class CoreApplication(ged.EventHelper):
@@ -121,6 +118,9 @@ class CoreApplication(ged.EventHelper):
             app.nicks[account] = app.settings.get_account_setting(account,
                                                                   'name')
 
+        from gajim.plugins.pluginmanager import PluginManager
+        from gajim.plugins.repository import PluginRepository
+
         app.plugin_manager = PluginManager()
         app.plugin_manager.init_plugins()
         app.plugin_repository = PluginRepository()
@@ -211,7 +211,7 @@ class CoreApplication(ged.EventHelper):
         self._profiling_session.disable()
         self._log.info('End profiling')
         ps = pstats.Stats(self._profiling_session)
-        ps = ps.sort_stats(SortKey.CUMULATIVE)
+        ps = ps.sort_stats(SortKey.TIME)
         ps.print_stats()
 
     def start_shutdown(self, *args: Any, **kwargs: Any) -> None:
@@ -258,12 +258,12 @@ class CoreApplication(ged.EventHelper):
         import warnings
         os.environ['GAJIM_LEAK'] = 'true'
 
-        def warn_with_traceback(message: Union[Warning, str],
+        def warn_with_traceback(message: Warning | str,
                                 category: type[Warning],
                                 filename: str,
                                 lineno: int,
-                                _file: Optional[TextIO] = None,
-                                line: Optional[str] = None) -> None:
+                                _file: TextIO | None = None,
+                                line: str | None = None) -> None:
 
             traceback.print_stack(file=sys.stderr)
             sys.stderr.write(warnings.formatwarning(message, category,
@@ -313,10 +313,10 @@ class CoreApplication(ged.EventHelper):
                      callback=self._on_update_response)
 
     def _on_update_response(self, request: HTTPRequest) -> None:
-
         if not request.is_complete():
-            self._log.warning('Error while performing gajim update check: %s',
-                              request.get_error(), request.get_status())
+            self._log.warning(
+                'Error while performing gajim update check: %s (%s)',
+                request.get_error_string(), request.get_status())
             return
 
         now = datetime.now()
@@ -324,9 +324,13 @@ class CoreApplication(ged.EventHelper):
 
         data = json.loads(request.get_data())
         latest_version = data['current_version']
+        setup_url = data['current_win_setup']
+        if IS_PORTABLE:
+            setup_url = data['current_win_portable_setup']
 
         if V(latest_version) > V(app.version):
-            app.ged.raise_event(GajimUpdateAvailable(version=latest_version))
+            app.ged.raise_event(GajimUpdateAvailable(version=latest_version,
+                                                     setup_url=setup_url))
             return
 
         self._log.info('Gajim is up to date')
@@ -348,7 +352,7 @@ class CoreApplication(ged.EventHelper):
             username = 'anon'
             account_label = f'anon@{domain}'
 
-        config: dict[str, Union[str, int, bool]] = {
+        config: dict[str, str | int | bool] = {
             'name': username,
             'resource': f'gajim.{get_random_string(8)}',
             'account_label': account_label,
@@ -356,7 +360,7 @@ class CoreApplication(ged.EventHelper):
             'anonymous_auth': anonymous,
         }
 
-        if proxy_name is not None:
+        if proxy_name:
             config['proxy'] = proxy_name
 
         use_custom_host = custom_host is not None
@@ -423,7 +427,7 @@ class CoreApplication(ged.EventHelper):
 
     def change_status(self,
                       status: str,
-                      account: Optional[str] = None
+                      account: str | None = None
                       ) -> None:
 
         if account is not None:
