@@ -13,25 +13,23 @@
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
 from typing import Any
-from typing import Callable
+from typing import cast
 from typing import Literal
 from typing import NamedTuple
-from typing import Optional
-from typing import Union
 from typing import overload
-from typing import cast
 from typing import TypedDict
 
-import sys
+import inspect
 import json
-import uuid
 import logging
 import sqlite3
-import inspect
+import sys
+import uuid
 import weakref
-from pathlib import Path
-from collections import namedtuple
 from collections import defaultdict
+from collections import namedtuple
+from collections.abc import Callable
+from pathlib import Path
 
 from gi.repository import GLib
 from nbxmpp.protocol import JID
@@ -40,48 +38,49 @@ from gajim import IS_PORTABLE
 from gajim.common import app
 from gajim.common import configpaths
 from gajim.common import optparser
-from gajim.common.storage.base import Encoder
-from gajim.common.storage.base import json_decoder
-from gajim.common.setting_values import WorkspaceSettings
-from gajim.common.setting_values import APP_SETTINGS
+from gajim.common.setting_values import ACCOUNT_SETTINGS
 from gajim.common.setting_values import AllContactSettings
 from gajim.common.setting_values import AllContactSettingsT
-from gajim.common.setting_values import BoolContactSettings
-from gajim.common.setting_values import StringContactSettings
 from gajim.common.setting_values import AllGroupChatSettings
 from gajim.common.setting_values import AllGroupChatSettingsT
-from gajim.common.setting_values import BoolGroupChatSettings
-from gajim.common.setting_values import IntGroupChatSettings
-from gajim.common.setting_values import StringGroupChatSettings
+from gajim.common.setting_values import AllSettingsT
 from gajim.common.setting_values import AllWorkspaceSettings
 from gajim.common.setting_values import AllWorkspaceSettingsT
-from gajim.common.setting_values import OpenChatsSettingT
-from gajim.common.setting_values import StringWorkspaceSettings
-from gajim.common.setting_values import ACCOUNT_SETTINGS
-from gajim.common.setting_values import PROXY_SETTINGS
-from gajim.common.setting_values import PROXY_EXAMPLES
-from gajim.common.setting_values import INITAL_WORKSPACE
-from gajim.common.setting_values import PLUGIN_SETTINGS
-from gajim.common.setting_values import WORKSPACE_SETTINGS
-from gajim.common.setting_values import DEFAULT_SOUNDEVENT_SETTINGS
-from gajim.common.setting_values import STATUS_PRESET_SETTINGS
-from gajim.common.setting_values import STATUS_PRESET_EXAMPLES
-from gajim.common.setting_values import HAS_APP_DEFAULT
-from gajim.common.setting_values import HAS_ACCOUNT_DEFAULT
-from gajim.common.setting_values import AllSettingsT
-from gajim.common.setting_values import IntSettings
-from gajim.common.setting_values import IntAccountSettings
-from gajim.common.setting_values import StringSettings
-from gajim.common.setting_values import StringAccountSettings
-from gajim.common.setting_values import BoolSettings
+from gajim.common.setting_values import APP_SETTINGS
 from gajim.common.setting_values import BoolAccountSettings
+from gajim.common.setting_values import BoolContactSettings
+from gajim.common.setting_values import BoolGroupChatSettings
+from gajim.common.setting_values import BoolSettings
+from gajim.common.setting_values import DEFAULT_SOUNDEVENT_SETTINGS
+from gajim.common.setting_values import FloatSettings
+from gajim.common.setting_values import HAS_ACCOUNT_DEFAULT
+from gajim.common.setting_values import HAS_APP_DEFAULT
+from gajim.common.setting_values import INITAL_WORKSPACE
+from gajim.common.setting_values import IntAccountSettings
+from gajim.common.setting_values import IntGroupChatSettings
+from gajim.common.setting_values import IntSettings
+from gajim.common.setting_values import OpenChatsSettingT
+from gajim.common.setting_values import PLUGIN_SETTINGS
+from gajim.common.setting_values import PROXY_EXAMPLES
+from gajim.common.setting_values import PROXY_SETTINGS
+from gajim.common.setting_values import STATUS_PRESET_EXAMPLES
+from gajim.common.setting_values import STATUS_PRESET_SETTINGS
+from gajim.common.setting_values import StringAccountSettings
+from gajim.common.setting_values import StringContactSettings
+from gajim.common.setting_values import StringGroupChatSettings
+from gajim.common.setting_values import StringSettings
+from gajim.common.setting_values import StringWorkspaceSettings
+from gajim.common.setting_values import WORKSPACE_SETTINGS
+from gajim.common.setting_values import WorkspaceSettings
+from gajim.common.storage.base import Encoder
+from gajim.common.storage.base import json_decoder
 
-SETTING_TYPE = Union[bool, int, str, object]
+SETTING_TYPE = bool | int | str | object
 
 
 log = logging.getLogger('gajim.c.settings')
 
-CURRENT_USER_VERSION = 3
+CURRENT_USER_VERSION = 5
 
 CREATE_SQL = '''
     CREATE TABLE settings (
@@ -94,22 +93,22 @@ CREATE_SQL = '''
             settings TEXT
     );
 
-    INSERT INTO settings(name, settings) VALUES ('app', '{}');
-    INSERT INTO settings(name, settings) VALUES ('soundevents', '{}');
-    INSERT INTO settings(name, settings) VALUES ('status_presets', '%s');
-    INSERT INTO settings(name, settings) VALUES ('proxies', '%s');
-    INSERT INTO settings(name, settings) VALUES ('plugins', '{}');
-    INSERT INTO settings(name, settings) VALUES ('workspaces', '%s');
+    INSERT INTO settings(name, settings) VALUES ('app', '{{}}');
+    INSERT INTO settings(name, settings) VALUES ('soundevents', '{{}}');
+    INSERT INTO settings(name, settings) VALUES ('status_presets', '{status}');
+    INSERT INTO settings(name, settings) VALUES ('proxies', '{proxies}');
+    INSERT INTO settings(name, settings) VALUES ('plugins', '{{}}');
+    INSERT INTO settings(name, settings) VALUES ('workspaces', '{workspaces}');
 
-    PRAGMA user_version=%s;
-    ''' % (json.dumps(STATUS_PRESET_EXAMPLES),
-           json.dumps(PROXY_EXAMPLES),
-           json.dumps(INITAL_WORKSPACE),
-           CURRENT_USER_VERSION)
+    PRAGMA user_version={version};
+    '''.format(status=json.dumps(STATUS_PRESET_EXAMPLES),
+               proxies=json.dumps(PROXY_EXAMPLES),
+               workspaces=json.dumps(INITAL_WORKSPACE),
+               version=CURRENT_USER_VERSION)
 
 
-_SignalCallable = Callable[[Any, str, Optional[str], Optional[JID]], Any]
-_CallbackDict = dict[tuple[str, Optional[str], Optional[JID]],
+_SignalCallable = Callable[[Any, str, str | None, JID | None], Any]
+_CallbackDict = dict[tuple[str, str | None, JID | None],
                      list[weakref.WeakMethod[_SignalCallable]]]
 
 if app.is_flatpak():
@@ -129,22 +128,23 @@ class SettingsDictT(TypedDict):
 
 
 class Settings:
-    def __init__(self):
+    def __init__(self, in_memory: bool = False):
         self._con = cast(sqlite3.Connection, None)
         self._commit_scheduled = None
+        self._in_memory = in_memory
 
         self._settings: SettingsDictT = {}
         self._app_overrides: dict[str, AllSettingsT] = {}
         self._account_settings: dict[
-            str, Union[Any, dict[str, dict[Union[JID, str], Any]]]] = {}
+            str, Any | dict[str, dict[JID | str, Any]]] = {}
 
         self._callbacks: _CallbackDict = defaultdict(list)
 
     def connect_signal(self,
                        setting: str,
                        func: _SignalCallable,
-                       account: Optional[str] = None,
-                       jid: Optional[JID] = None) -> None:
+                       account: str | None = None,
+                       jid: JID | None = None) -> None:
         if not inspect.ismethod(func):
             # static methods are not bound to an object so we can’t easily
             # remove the func once it should not be called anymore
@@ -154,7 +154,7 @@ class Settings:
         self._callbacks[(setting, account, jid)].append(weak_func)
 
     def disconnect_signals(self, object_: object) -> Any:
-        for _, handlers in self._callbacks.items():
+        for handlers in self._callbacks.values():
             for handler in list(handlers):
                 if isinstance(handler, tuple):
                     continue
@@ -166,10 +166,10 @@ class Settings:
                     setting: str,
                     widget: Any,
                     func_name: str,
-                    account: Optional[str] = None,
-                    jid: Optional[JID] = None,
+                    account: str | None = None,
+                    jid: JID | None = None,
                     inverted: bool = False,
-                    default_text: Optional[str] = None
+                    default_text: str | None = None
                     ) -> None:
 
         callbacks = self._callbacks[(setting, account, jid)]
@@ -184,8 +184,8 @@ class Settings:
     def _notify(self,
                 value: Any,
                 setting: str,
-                account: Optional[str] = None,
-                jid: Optional[JID] = None) -> None:
+                account: str | None = None,
+                jid: JID | None = None) -> None:
 
         log.info('Signal: %s changed', setting)
 
@@ -220,7 +220,10 @@ class Settings:
 
     def init(self) -> None:
         self._setup_installation_defaults()
-        self._connect_database()
+        if self._in_memory:
+            self._connect_in_memory_database()
+        else:
+            self._connect_database()
         self._load_settings()
         self._load_account_settings()
         if not self._settings['app']:
@@ -237,6 +240,9 @@ class Settings:
         if IS_PORTABLE:
             APP_SETTINGS['use_keyring'] = False
 
+        if sys.platform == 'win32':
+            APP_SETTINGS['app_font_size'] = 1.125
+
     def _load_app_overrides(self) -> None:
         if not OVERRIDES_PATH.exists():
             return
@@ -254,7 +260,7 @@ class Settings:
     def _namedtuple_factory(cursor: sqlite3.Cursor,
                             row: tuple[Any, ...]) -> NamedTuple:
         fields = [col[0] for col in cursor.description]
-        return namedtuple('Row', fields)(*row)  # type: ignore
+        return namedtuple('Row', fields)(*row)  # pyright: ignore
 
     def _connect_database(self) -> None:
         path = configpaths.get('SETTINGS')
@@ -268,6 +274,20 @@ class Settings:
         self._con = sqlite3.connect(path)
         self._con.row_factory = self._namedtuple_factory
 
+    def _connect_in_memory_database(self) -> None:
+        log.info('Creating in memory')
+        self._con = sqlite3.connect(':memory:')
+        self._con.row_factory = self._namedtuple_factory
+
+        try:
+            self._con.executescript(CREATE_SQL)
+        except Exception:
+            log.exception('Error')
+            self._con.close()
+            sys.exit()
+
+        self._con.commit()
+
     @staticmethod
     def _create_database(statement: str, path: Path) -> None:
         log.info('Creating %s', path)
@@ -313,6 +333,9 @@ class Settings:
         self._con.commit()
 
     def _migrate_database(self) -> None:
+        if self._in_memory:
+            return
+
         try:
             self._migrate()
         except Exception:
@@ -365,7 +388,14 @@ class Settings:
 
             self._set_user_version(4)
 
+        if version < 5:
+            self._settings['app'].pop('muclumbus_api_http_uri', None)
+            self._set_user_version(5)
+
     def _migrate_old_config(self) -> None:
+        if self._in_memory:
+            return
+
         config_file = configpaths.get('CONFIG_FILE')
         if not config_file.exists():
             return
@@ -414,12 +444,12 @@ class Settings:
         # Migrate deprecated settings
         value = app_settings.pop('send_chatstate_muc_default', None)
         if value is not None:
-            for account, settings in self._account_settings.items():
+            for settings in self._account_settings.values():
                 settings['account']['gc_send_chatstate_default'] = value
 
         value = app_settings.pop('send_chatstate_default', None)
         if value is not None:
-            for account, settings in self._account_settings.items():
+            for settings in self._account_settings.values():
                 settings['account']['send_chatstate_default'] = value
 
         value = app_settings.pop('print_join_left_default', None)
@@ -476,8 +506,8 @@ class Settings:
                     jid]['encryption'] = encryption
             self._commit_account_settings(account)
 
-    def _split_encryption_config_key(self, key: str) -> tuple[Optional[str],
-                                                              Optional[str]]:
+    def _split_encryption_config_key(self, key: str) -> tuple[str | None,
+                                                              str | None]:
         for account in self._account_settings:
             if not key.startswith(account):
                 continue
@@ -602,6 +632,10 @@ class Settings:
     def get_app_setting(self, setting: IntSettings) -> int:
         ...
 
+    @overload
+    def get_app_setting(self, setting: FloatSettings) -> float:
+        ...
+
     def get_app_setting(self, setting: str) -> AllSettingsT:
         if setting not in APP_SETTINGS:
             raise ValueError(f'Invalid app setting: {setting}')
@@ -616,19 +650,25 @@ class Settings:
     @overload
     def set_app_setting(self,
                         setting: BoolSettings,
-                        value: Optional[bool]) -> None:
+                        value: bool | None) -> None:
         ...
 
     @overload
     def set_app_setting(self,
                         setting: StringSettings,
-                        value: Optional[str]) -> None:
+                        value: str | None) -> None:
         ...
 
     @overload
     def set_app_setting(self,
                         setting: IntSettings,
-                        value: Optional[int]) -> None:
+                        value: int | None) -> None:
+        ...
+
+    @overload
+    def set_app_setting(self,
+                        setting: FloatSettings,
+                        value: float | None) -> None:
         ...
 
     @overload
@@ -639,7 +679,7 @@ class Settings:
 
     def set_app_setting(self,
                         setting: str,
-                        value: Optional[AllSettingsT]) -> None:
+                        value: AllSettingsT | None) -> None:
 
         if setting not in APP_SETTINGS:
             raise ValueError(f'Invalid app setting: {setting}')
@@ -779,27 +819,27 @@ class Settings:
     def set_account_setting(self,
                             account: str,
                             setting: StringAccountSettings,
-                            value: Optional[str]) -> None:
+                            value: str | None) -> None:
         ...
 
     @overload
     def set_account_setting(self,
                             account: str,
                             setting: IntAccountSettings,
-                            value: Optional[int]) -> None:
+                            value: int | None) -> None:
         ...
 
     @overload
     def set_account_setting(self,
                             account: str,
                             setting: BoolAccountSettings,
-                            value: Optional[bool]) -> None:
+                            value: bool | None) -> None:
         ...
 
     def set_account_setting(self,
                             account: str,
                             setting: str,
-                            value: Optional[AllSettingsT]) -> None:
+                            value: AllSettingsT | None) -> None:
 
         if account not in self._account_settings:
             raise ValueError(f'Account missing: {account}')
@@ -898,7 +938,7 @@ class Settings:
                                account: str,
                                jid: JID,
                                setting: StringGroupChatSettings,
-                               value: str) -> None:
+                               value: str | None) -> None:
         ...
 
     @overload
@@ -906,7 +946,7 @@ class Settings:
                                account: str,
                                jid: JID,
                                setting: IntGroupChatSettings,
-                               value: int) -> None:
+                               value: int | None) -> None:
         ...
 
     @overload
@@ -914,14 +954,14 @@ class Settings:
                                account: str,
                                jid: JID,
                                setting: BoolGroupChatSettings,
-                               value: bool) -> None:
+                               value: bool | None) -> None:
         ...
 
     def set_group_chat_setting(self,
                                account: str,
                                jid: JID,
                                setting: AllGroupChatSettings,
-                               value: AllGroupChatSettingsT) -> None:
+                               value: AllGroupChatSettingsT | None) -> None:
 
         if account not in self._account_settings:
             raise ValueError(f'Account missing: {account}')
@@ -976,7 +1016,7 @@ class Settings:
     def set_group_chat_settings(self,
                                 setting: str,
                                 value: SETTING_TYPE,
-                                context: Optional[str] = None
+                                context: str | None = None
                                 ) -> None:
 
         for account, acc_settings in self._account_settings.items():
@@ -1033,7 +1073,7 @@ class Settings:
                             account: str,
                             jid: JID,
                             setting: StringContactSettings,
-                            value: str) -> None:
+                            value: str | None) -> None:
         ...
 
     @overload
@@ -1041,14 +1081,14 @@ class Settings:
                             account: str,
                             jid: JID,
                             setting: BoolContactSettings,
-                            value: bool) -> None:
+                            value: bool | None) -> None:
         ...
 
     def set_contact_setting(self,
                             account: str,
                             jid: JID,
                             setting: AllContactSettings,
-                            value: AllContactSettingsT) -> None:
+                            value: AllContactSettingsT | None) -> None:
 
         if account not in self._account_settings:
             raise ValueError(f'Account missing: {account}')
@@ -1319,6 +1359,15 @@ class Settings:
         del self._settings['workspaces'][id_]
         self._commit_settings('workspaces')
 
+    def shutdown(self) -> None:
+        if self._commit_scheduled is not None:
+            GLib.source_remove(self._commit_scheduled)
+            self._commit_scheduled = None
+
+        self._commit()
+        self._con.close()
+        del self._con
+
 
 class LegacyConfig:
 
