@@ -17,12 +17,11 @@
 
 from __future__ import annotations
 
-from typing import Callable
 from typing import cast
-from typing import Iterator
-from typing import Optional
 
 import textwrap
+from collections.abc import Callable
+from collections.abc import Iterator
 from datetime import datetime
 from urllib.parse import quote
 
@@ -43,8 +42,10 @@ from gajim.common.helpers import jid_is_blocked
 from gajim.common.i18n import _
 from gajim.common.i18n import get_short_lang_code
 from gajim.common.i18n import p_
+from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import can_add_to_roster
 from gajim.common.modules.contacts import GroupchatContact
+from gajim.common.modules.contacts import GroupchatParticipant
 from gajim.common.preview import Preview
 from gajim.common.structs import URI
 from gajim.common.util.text import escape_iri_path_segment
@@ -96,6 +97,12 @@ def get_singlechat_menu(contact: types.BareContact) -> GajimMenu:
         params = AccountJidParam(account=account, jid=contact.jid)
         menu.add_item(_('Add to Contact List…'), 'win.add-to-roster', params)
 
+    menu.add_item(
+        _('Execute Command…'),
+        f'app.{account}-execute-command',
+        contact.jid.bare,
+    )
+
     return menu
 
 
@@ -199,7 +206,7 @@ def build_accounts_menu() -> None:
                                             'app.add-contact::')
         acc_menu.insert_item(1, add_contact_item)
         menubar.remove(menu_position)
-        menubar.insert_submenu(menu_position, _('Accounts'), acc_menu)
+        menubar.insert_submenu(menu_position, _('_Accounts'), acc_menu)
 
 
 def get_encryption_menu() -> GajimMenu:
@@ -494,6 +501,8 @@ def get_chat_list_row_menu(workspace_id: str,
 
     client = app.get_client(account)
     contact = client.get_module('Contacts').get_contact(jid)
+    assert isinstance(
+        contact, BareContact | GroupchatContact | GroupchatParticipant)
 
     menu = GajimMenu()
 
@@ -655,7 +664,7 @@ def get_groupchat_participant_menu(account: str,
     return menu
 
 
-def get_component_search_menu(jid: Optional[str], copy_text: str) -> GajimMenu:
+def get_component_search_menu(jid: str | None, copy_text: str) -> GajimMenu:
     menuitems: MenuItemListT = [
         (_('Copy'), 'app.copy-text', copy_text),
     ]
@@ -672,9 +681,9 @@ def get_chat_row_menu(contact: types.ChatContactT,
                       name: str,
                       text: str,
                       timestamp: datetime,
-                      message_id: Optional[str],
-                      stanza_id: Optional[str],
-                      log_line_id: Optional[int]
+                      message_id: str | None,
+                      stanza_id: str | None,
+                      log_line_id: int | None
                       ) -> GajimMenu:
 
     menu_items: MenuItemListT = []
@@ -806,7 +815,7 @@ def get_workspace_menu(workspace_id: str) -> GajimMenu:
     return GajimMenu.from_list(menuitems)
 
 
-def escape_mnemonic(label: Optional[str]) -> Optional[str]:
+def escape_mnemonic(label: str | None) -> str | None:
     if label is None:
         return None
     # Underscore inside a label means the next letter is a keyboard
