@@ -17,34 +17,33 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Callable
-from typing import Optional
-from typing import Union
 from typing import cast
+from typing import get_args
 
 import logging
+from collections.abc import Callable
 
-from gi.repository import Gtk
-from gi.repository import GLib
 from gi.repository import Gdk
+from gi.repository import GLib
+from gi.repository import Gtk
 from gi.repository import Pango
-
 from nbxmpp.protocol import JID
 
 from gajim.common import app
 from gajim.common import passwords
 from gajim.common.i18n import _
-from gajim.common.i18n import Q_
+from gajim.common.i18n import p_
 from gajim.common.setting_values import AllSettingsT
+from gajim.common.setting_values import FloatSettings
 
-from .util import get_image_button
-from .util import MaxWidthComboBoxText
-from .util import open_window
-from .const import Setting
-from .const import SettingKind
-from .const import SettingType
+from gajim.gtk.const import Setting
+from gajim.gtk.const import SettingKind
+from gajim.gtk.const import SettingType
+from gajim.gtk.util import get_image_button
+from gajim.gtk.util import MaxWidthComboBoxText
+from gajim.gtk.util import open_window
 
-log = logging.getLogger('gajim.gui.settings')
+log = logging.getLogger('gajim.gtk.settings')
 
 
 class SettingsDialog(Gtk.ApplicationWindow):
@@ -54,7 +53,7 @@ class SettingsDialog(Gtk.ApplicationWindow):
                  flags: Gtk.DialogFlags,
                  settings: list[Setting],
                  account: str,
-                 extend: Optional[dict[SettingKind, GenericSetting]] = None
+                 extend: dict[SettingKind, GenericSetting] | None = None
                  ) -> None:
         Gtk.ApplicationWindow.__init__(self)
         self.set_application(app.app)
@@ -98,9 +97,9 @@ class SettingsDialog(Gtk.ApplicationWindow):
 
 class SettingsBox(Gtk.ListBox):
     def __init__(self,
-                 account: Optional[str] = None,
-                 jid: Optional[str] = None,
-                 extend: Optional[dict[SettingKind, GenericSetting]] = None
+                 account: str | None = None,
+                 jid: str | None = None,
+                 extend: dict[SettingKind, GenericSetting] | None = None
                  ) -> None:
         Gtk.ListBox.__init__(self)
         self.get_style_context().add_class('settings-box')
@@ -177,13 +176,13 @@ class GenericSetting(Gtk.ListBoxRow):
                  label: str,
                  type_: SettingType,
                  value: AllSettingsT,
-                 name: Optional[str],
-                 callback: Optional[Callable[..., None]] = None,
-                 data: Optional[Any] = None,
-                 desc: Optional[str] = None,
-                 bind: Optional[str] = None,
+                 name: str | None,
+                 callback: Callable[..., None] | None = None,
+                 data: Any | None = None,
+                 desc: str | None = None,
+                 bind: str | None = None,
                  inverted: bool = False,
-                 enabled_func: Optional[Callable[..., bool]] = None
+                 enabled_func: Callable[..., bool] | None = None
                  ) -> None:
 
         Gtk.ListBoxRow.__init__(self)
@@ -279,8 +278,8 @@ class GenericSetting(Gtk.ListBoxRow):
 
     def _parse_bind(self) -> tuple[SettingType,
                                    str,
-                                   Optional[str],
-                                   Optional[JID]]:
+                                   str | None,
+                                   JID | None]:
         assert self.bind is not None
         if '::' not in self.bind:
             return SettingType.CONFIG, self.bind, None, None
@@ -307,7 +306,7 @@ class GenericSetting(Gtk.ListBoxRow):
                     value: AllSettingsT,
                     account: str,
                     jid: JID
-                    ) -> Optional[AllSettingsT]:
+                    ) -> AllSettingsT | None:
         if value is None:
             return None
         if type_ == SettingType.VALUE:
@@ -379,9 +378,7 @@ class GenericSetting(Gtk.ListBoxRow):
         self.set_sensitive(enabled_func_value)
 
     def _add_action_button(self,
-                           kwargs: dict[str, Union[str,
-                                                   Callable[..., None],
-                                                   None]]
+                           kwargs: dict[str, str | Callable[..., None] | None]
                            ) -> None:
         icon_name = cast(str, kwargs.get('button-icon-name'))
         button_text = cast(str, kwargs.get('button-text'))
@@ -450,7 +447,7 @@ class SwitchSetting(GenericSetting):
         self._set_label(value)
 
     def _set_label(self, active: bool) -> None:
-        text = Q_('?switch:On') if active else Q_('?switch:Off')
+        text = p_('Switch', 'On') if active else p_('Switch', 'Off')
         self._switch_state_label.set_text(text)
 
 
@@ -567,14 +564,14 @@ class DialogSetting(GenericSetting):
 
 
 class SpinSetting(GenericSetting):
-    def __init__(self, *args: Any, range_: tuple[float, float]) -> None:
+    def __init__(self, *args: Any, range_: tuple[float, float, float]) -> None:
         GenericSetting.__init__(self, *args)
 
-        lower, upper = range_
+        lower, upper, step = range_
         adjustment = Gtk.Adjustment(value=0,
                                     lower=lower,
                                     upper=upper,
-                                    step_increment=1,
+                                    step_increment=step,
                                     page_increment=10,
                                     page_size=0)
 
@@ -584,6 +581,9 @@ class SpinSetting(GenericSetting):
         self.spin.set_update_policy(Gtk.SpinButtonUpdatePolicy.IF_VALID)
 
         assert self.setting_value is not None
+        if isinstance(self.setting_value, float):
+            self.spin.set_digits(3)
+
         self.spin.set_value(float(self.setting_value))
         self.spin.set_halign(Gtk.Align.FILL)
         self.spin.set_valign(Gtk.Align.CENTER)
@@ -591,13 +591,29 @@ class SpinSetting(GenericSetting):
 
         self.setting_box.pack_start(self.spin, True, True, 0)
 
+        assert isinstance(self.value, str)
+        app.settings.connect_signal(self.value,
+                                    self._on_setting_changed,
+                                    account=self.account,
+                                    jid=self.jid)
+        self.connect('destroy', self._on_destroy)
+
         self.show_all()
 
+    def _on_setting_changed(self, value: float, *args: Any) -> None:
+        self.spin.set_value(value)
+
+    def _on_destroy(self, *args: Any) -> None:
+        app.settings.disconnect_signals(self)
+
     def on_row_activated(self) -> None:
         self.spin.grab_focus()
 
     def on_value_change(self, spin: Gtk.SpinButton, *args: Any) -> None:
-        value = spin.get_value_as_int()
+        if self.value in list(get_args(FloatSettings)):
+            value = spin.get_value()
+        else:
+            value = spin.get_value_as_int()
         self.set_value(value)
 
 
@@ -700,7 +716,7 @@ class LoginSetting(DialogSetting):
 class PopoverSetting(GenericSetting):
     def __init__(self,
                  *args: Any,
-                 entries: Union[list[str], dict[str, str]],
+                 entries: list[str] | dict[str, str],
                  **kwargs: Any
                  ) -> None:
         GenericSetting.__init__(self, *args)
@@ -762,7 +778,7 @@ class PopoverSetting(GenericSetting):
         self.show_all()
 
     @staticmethod
-    def _convert_to_dict(entries: Union[list[Any], dict[Any, Any]]
+    def _convert_to_dict(entries: list[Any] | dict[Any, Any]
                          ) -> dict[Any, Any]:
         if isinstance(entries, list):
             entries = {key: key for key in entries}
@@ -796,7 +812,7 @@ class PopoverSetting(GenericSetting):
     def on_row_activated(self) -> None:
         self._popover.popup()
 
-    def update_entries(self, entries: Union[list[str], dict[str, str]]) -> None:
+    def update_entries(self, entries: list[str] | dict[str, str]) -> None:
         self._entries = self._convert_to_dict(entries)
         self._menu_listbox.foreach(self._menu_listbox.remove)
         self._add_menu_entries()
@@ -820,7 +836,7 @@ class PopoverRow(Gtk.ListBoxRow):
 class ComboSetting(GenericSetting):
     def __init__(self,
                  *args: Any,
-                 combo_items: list[Union[str, tuple[str, str]]]
+                 combo_items: list[str | tuple[str, str]]
                  ) -> None:
         GenericSetting.__init__(self, *args)
 
@@ -871,7 +887,7 @@ class CutstomHostnameSetting(DialogSetting):
     def get_setting_value(self) -> str:
         custom = app.settings.get_account_setting(self.account,
                                                   'use_custom_host')
-        return Q_('?switch:On') if custom else Q_('?switch:Off')
+        return p_('Switch', 'On') if custom else p_('Switch', 'Off')
 
 
 class ChangePasswordSetting(DialogSetting):
@@ -897,7 +913,7 @@ class CutstomAutoAwaySetting(DialogSetting):
 
     def get_setting_value(self) -> str:
         value = app.settings.get('autoaway')
-        return Q_('?switch:On') if value else Q_('?switch:Off')
+        return p_('Switch', 'On') if value else p_('Switch', 'Off')
 
 
 class CutstomAutoExtendedAwaySetting(DialogSetting):
@@ -906,7 +922,7 @@ class CutstomAutoExtendedAwaySetting(DialogSetting):
 
     def get_setting_value(self) -> str:
         value = app.settings.get('autoxa')
-        return Q_('?switch:On') if value else Q_('?switch:Off')
+        return p_('Switch', 'On') if value else p_('Switch', 'Off')
 
 
 class CustomStunServerSetting(DialogSetting):
@@ -915,7 +931,7 @@ class CustomStunServerSetting(DialogSetting):
 
     def get_setting_value(self) -> str:
         value = app.settings.get('use_stun_server')
-        return Q_('?switch:On') if value else Q_('?switch:Off')
+        return p_('Switch', 'On') if value else p_('Switch', 'Off')
 
 
 class NotificationsSetting(DialogSetting):
@@ -924,4 +940,4 @@ class NotificationsSetting(DialogSetting):
 
     def get_setting_value(self) -> str:
         value = app.settings.get('show_notifications')
-        return Q_('?switch:On') if value else Q_('?switch:Off')
+        return p_('Switch', 'On') if value else p_('Switch', 'Off')
